//
//  ViewController.m
//  BTDemo
//
//  Created by etek on 18/9/11.
//  Copyright (c) 2018年 ETEK. All rights reserved.
//

#import "ViewController.h"


@interface ViewController () {
    NSMutableArray *tableDataArr;
    BleWrapper     *bleWrapper;  // BLE class
    Boolean         bScanning;
}
@property (weak, nonatomic) IBOutlet UITableView *mDeviceList;
@property (weak, nonatomic) IBOutlet UISegmentedControl *ctrlScanBt;

@property (strong, nonatomic) BTDevViewController* viewBTDevController;


@end



@implementation ViewController

@synthesize viewBTDevController;

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Do any additional setup after loading the view, typically from a nib.
    
    // inital Ble class
    bScanning = false;
    bleWrapper = [[BleWrapper alloc] init];
    bleWrapper.mDelegate = self;
    tableDataArr = [bleWrapper getBTDevArrays];
    _ctrlScanBt.selectedSegmentIndex = 1;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)onScanBtDevices:(id)sender {
    if ( [_ctrlScanBt selectedSegmentIndex] == 0 ) {
        bScanning = true;
        [bleWrapper StartScan];
    }
    else if ( [_ctrlScanBt selectedSegmentIndex] == 1 ) {
        bScanning = false;
        [bleWrapper StopScan];
    }
}

#pragma mark ---> TableView

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return tableDataArr.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    DeviceInfoCell *cell = [DeviceInfoCell cellWithTableCell:tableView tag:(int)[indexPath row]];
    
    BTInfo* info = [tableDataArr objectAtIndex:(int)[indexPath row]];
    
    if ( info ) {
        
        NSString* str;
        for (int i=0; i<info->mBufLen; i++ ) {
            NSString* temp = [[NSString alloc] initWithFormat:@"%02X",info->mScanBuf[i]];
            if ( i == 0 ) {
                str = temp;
            }
            else
                str = [[NSString alloc] initWithFormat:@"%@ %@",str,temp];
        }
        cell.mDelegate = self;
        [cell updateCellInfo:[info->mPer name] mac:info->mParam rxinfo:str];
    }
    
    return cell;
}

#pragma mark --> DeviceInfoCellDelegate

- (void)onButtonClick:(int)index {
    
    if ( bleWrapper == nil ) return;
    [bleWrapper connectBleDevice:index];
}

#pragma mark --> BTDevViewControllerDelegate

- (void) disconnectDevice:(int)index {
    if ( bleWrapper == nil ) return;
    [bleWrapper disconnectBleDevice:index];
}

- (void)onSendBuffer:(int)index buffer:(Byte*)buffer lenght:(int)lenght {
    if ( bleWrapper == nil ) return;
    [bleWrapper sendDataToBleById:index buffer:buffer lenght:lenght];
}

#pragma mark --> BleWrapper

- (void)onRecieviceData:(DevData*)data {
    if ( viewBTDevController == nil ) return;
    NSString* str;
    for (int i=0; i<data->mLenght; i++ ) {
        NSString* temp = [[NSString alloc] initWithFormat:@"%02X",data->mData[i]];
        if ( i == 0 ) {
            str = temp;
        }
        else
            str = [[NSString alloc] initWithFormat:@"%@ %@",str,temp];
    }
    // NSLog(@"%@",str);
    [viewBTDevController updateRxData:str];
}

- (void)onWriteDataOk:(DevData*)data {
    
}

- (void)onCharacteristicInfo:(DevCharacteristic*)chara {
    
}

- (void)onBTConnectState:(CBPeripheral*)peri state:(int)state; {
    NSString* str;
    switch (state) {
        case DEVStateConnettng:
            str = @"Connectting";
            break;
        case DEVStateConnected:
            str = @"GetDeviceInfo";
            break;
        case DEVStateDisconneced:
            str = @"Disconnect";
            if ( viewBTDevController == nil ) break;
            [viewBTDevController exitBTDevView];
            break;
        case DEVStateDeviceOk: {
            str = @"Connected";
            
            [self performSelectorOnMainThread:@selector(openBTDevController:)
                                   withObject:peri
                                waitUntilDone:YES];
        } break;
            
        default:
            break;
    }
    [_mDeviceList reloadData];
}

- (void)onFoundNewBleDev:(ScanInfo*)info {
    [_mDeviceList reloadData];
}

- (void)onBTMacInfo:(DevData*)data {
    NSString* str;
    for (int i=0; i<data->mLenght; i++ ) {
        NSString* temp = [[NSString alloc] initWithFormat:@"%02X",data->mData[i]];
        if ( i == 0 ) {
            str = temp;
        }
        else
            str = [[NSString alloc] initWithFormat:@"%@ %@",str,temp];
    }
    NSLog(@"MAC: %@",str);
}

- (void)openBTDevController:(CBPeripheral*)per {
    if ( viewBTDevController == nil ) {
        UIStoryboard* storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]];
        viewBTDevController = [storyboard instantiateViewControllerWithIdentifier:@"BTDevViewController"];
    }
    
    viewBTDevController.mDelegate = self;
    viewBTDevController.mDevTag = [bleWrapper getBTDeviceIndex:per];
    viewBTDevController.mDevName = [per name];
    [viewBTDevController updateCtrl];
    
    
    if ( self.navigationController != nil ) {
        [self.navigationController pushViewController:viewBTDevController animated:YES];
    }
    
}

- (void)closeBTDevController {
    [self.navigationController popViewControllerAnimated:YES];
}


@end
